/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include "MPUcalculations.h"



#define STR_LEN 64
char str[STR_LEN+1] ;
int sys_tick_slot = 0 ;

void cls(void){
    UART_PutString("\033c") ; /* reset */
    CyDelay(100) ;
    UART_PutString("\033[2J") ; /* clear screen */
    CyDelay(100) ;
}

void splash(char *title){
    cls() ;
    if (title && *title) {
        UART_PutString(title) ;
        UART_PutString(" ") ;
    }
    snprintf(str, STR_LEN, "(%s %s)\n\r", __DATE__, __TIME__) ;
    UART_PutString(str) ;
}

volatile uint32_t tick_count = 0 ;
CY_ISR(tick_callback){
    tick_count++ ;
}

int find_empty_slot(void){
    int result = -1 ;
    uint32_t i ;
    for (i = 0 ; i < CY_SYS_SYST_NUM_OF_CALLBACKS ; i++ ) {
        if (CySysTickGetCallback(i) == NULL) {
            result = i ;
            break ;
        }
    }
    return(result) ;
}
        
void show_delta_time(void){
    static uint32_t prev_tick;
    uint32_t new_tick;
    uint32_t delta;
    
    
    new_tick = tick_count ;
    delta = new_tick - prev_tick ;
    prev_tick = new_tick ;
    //snprintf(str, STR_LEN, "Current: %d -> %d ms passed\n\r", new_tick, delta) ;
    //UART_PutString(str) ;

}

void clear_millis(void){
    tick_count = 0 ;
}

uint32_t millis(void){
    return( tick_count ) ;
}

void controlmilli(){
    int sys_tick_slot = 0 ;
    
    sys_tick_slot = find_empty_slot() ;
    if (sys_tick_slot < 0) {
        UART_PutString("Sorry No empty SysTick Slot available\n\r") ;
        while(1) { } /* halting here */
    } 
    else {
        CySysTickStart() ;
        CySysTickSetCallback(sys_tick_slot, tick_callback) ;
    }
}

int MPU6050_calculate_angle(void){
    
       //////////////////// get raw MPU values //////////////////
    
        MPU6050_GetRawAccelGyro(mybuf); // put values from MPU in mybuf             
        //printf("Raw data is: %d   %d  %d \r\n\r\n",mybuf[0],mybuf[1],mybuf[2]); // 0,1,2 => acceleration |3,4,5 => Angular rate
       
        /////////////////////////////////////////////////////////
        
        ///////////////// Get/process Acc values ////////////////
        
        AccX = mybuf[0]/16384.0; // X
        AccY = mybuf[1]/16384.0; // Y
        AccZ = mybuf[2]/16384.0; // Z
        //printf("Bewerkte date is        %f  %f  %f \r\n\r\n",AccX, AccY, AccZ);
        
        accAngleZ = atan2(AccZ, AccX)*(180/M_PI) - AccErrorZ; // gives Ø between Z- and X-axis
        //accAngleZ = (atan2f(AccZ,sqrt(pow(AccX,2) + pow(AccY,2)))* 180/M_PI)-AccErrorZ;// Gives the angle Z axis and X
        //printf("AngleZ: %f \r\n",accAngleZ);
        
        /////////////////////////////////////////////////////////
        
        ///////////////// Get/process Gyro values ///////////////
        
        GyroX = mybuf[3]/16.4; // Current Gyro value
        GyroY = mybuf[4]/16.4;// Current Gyro value
        GyroZ = mybuf[5]/16.4; // Current Gyro value
        
        GyroX = GyroX - GyroErrorX; // Current gyro - error value from MPU6050
        GyroY = GyroY - GyroErrorY; // Current gyro - error value from MPU6050
        GyroZ = GyroZ - GyroErrorZ; // Current gyro - error value from MPU6050
        
        GyroRateX = GyroX - GyroX0; // get rate of change by Current Gyro - old Gyro
        GyroRateY = GyroY - GyroY0; // get rate of change by Current Gyro - old Gyro
        GyroRateZ = GyroZ - GyroZ0; // get rate of change by Current Gyro - old Gyro        
        //printf("gyroX: %f  GyroY: %f  GyroZ: %f\r\n",GyroRateX, GyroRateY, GyroRateZ);
        
        GyroX0 = GyroX; // Current Gyro is now previous Gyro value
        GyroY0 = GyroY; // Current Gyro is now previous Gyro value
        GyroZ0 = GyroZ; // Current Gyro is now previous Gyro value
        
        /////////////////////////////////////////////////////////
        
        /////////////////// Get cyclus time /////////////////////
        
        CurrentTime = millis();
        DeltaTime = CurrentTime - PreviousTime;
        PreviousTime = CurrentTime;
        //printf("\r\n The elapsed time is %f seconds of %f millisecondes\r\n",DeltaTime/1000,DeltaTime);
        
        /////////////////////////////////////////////////////////
        
        ////////////// Deg/s  * dT => Deg //////////////////////
        
        gyroAngleX = gyroAngleX + GyroRateX * (DeltaTime/1000.0); //Delta time is ms =>  ms /1000 = s
        gyroAngleY = gyroAngleY + GyroRateY * (DeltaTime/1000.0);
        gyroAngleZ = gyroAngleZ + GyroRateZ * (DeltaTime/1000.0);
        printf("\r\nGyroAngleY: %f  dT: %f ms\r\n", gyroAngleY, DeltaTime);
        
        /////////////////////////////////////////////////////////
        
        ////// Complumantary fileter to get a more 'stable angle' //////
        
        //get_Angle = (0.024 * gyroAngleY + 0.976 * accAngleZ); // cheat angle:=> is just the acc is not stable with sudden interference
        
        get_Angle = 0.98*(get_Angle+gyroAngleY) + 0.02*accAngleZ;
        //printf("%d \r\n", get_Angle);
        return get_Angle;
        
        
        /////////////////////////////////////////////////////////
}
void calculate_IMU_error() 
{
    
    printf("calculating error\r\n");
    int16_t corBuf[10];
    int c=0;
 
  /////////////// Generate error value for the acc///////////////
  while (c < 200) 
    {
    MPU6050_GetRawAccelGyro(corBuf);
    
    AccX = (corBuf[0])/16384.0; // X
    AccY = corBuf[1]/16384.0; // Y
    AccZ = corBuf[2]/16384.0; // Z
    /// Sum all readings ///
    AccErrorX = AccErrorX + ((atan((AccY) / sqrt(pow((AccX), 2) + pow((AccZ), 2))) * 180 / M_PI));
    AccErrorY = AccErrorY + ((atan(-1 * (AccX) / sqrt(pow((AccY), 2) + pow((AccZ), 2))) * 180 / M_PI));
    AccErrorZ = AccErrorZ + atan2(AccZ, AccX)*(180/M_PI);
    //AccErrorZ = AccErrorZ + atan2f(AccZ,sqrt(pow(AccX,2) + pow(AccY,2)))* 180/M_PI;

    c++;
    }
    
    AccErrorX = AccErrorX / 200.0;
    AccErrorY = AccErrorY / 200.0;
    AccErrorZ= AccErrorZ / 200.0;
    /////////////////////////////////////////////////////////////
    
    c = 0; // reset forloop counter
    
    ///////////// Generate error value for the gyro /////////////
  while (c < 200) 
    {
    MPU6050_GetRawAccelGyro(corBuf);
    /// Sum all readings ///
    GyroX = corBuf[3] / 16.4;
    GyroY = corBuf[4] / 16.4;
    GyroZ = corBuf[5] / 16.4;
    
    GyroErrorX = GyroErrorX + GyroX;
    GyroErrorY = GyroErrorY + GyroY;
    GyroErrorZ = GyroErrorZ + GyroZ;
    
    c++;
    }
    
    GyroErrorX = GyroErrorX / 200.0;
    GyroErrorY = GyroErrorY / 200.0;
    GyroErrorZ = GyroErrorZ / 200.0;
    /////////////////////////////////////////////////////////////
    
    /////////////////// Print the error values //////////////////
    printf("The error values are: \r\n");
    printf("AccErrorX:  %f  AccErrorY:  %f  AcceErrorZ: %f||  GyroErrorX: %f  GyroErrorY: %f  GyroErrorZ: %f\r\n",AccErrorX,AccErrorY, AccErrorZ,GyroErrorX,GyroErrorY,GyroErrorZ);
    
    /////////////////////////////////////////////////////////////
}
/* [] END OF FILE */
